
/*
   Socket Class for C++
   This source code is provided under the terms of the
   GNU general public license version 3. See gpl3.txt for details.

   thanks to
      http://www.zotteljedi.de/
      http://www.linuxhowtos.org/C_C++/socket.htm
*/


#include "socket.h"
using namespace std;

// ------------------------------------------ stuff regarding active sockets

namespace AlgorithMan
{
    void* MonitorSocket(void* s)
    {
        Socket* S = (Socket*) s;
        const int max_chars = 255;
        char *buffer = new(char[max_chars+1]);
        int n;
        
        while(S->active)
        {
            n = read(S->socket_handle, buffer, max_chars);
            if(n <= 0)
            {
                throw "ERROR: connection lost\n";
                S->active = false;
                S->Disconnect();
                break;
            }
            S->InternalOnRecieve(buffer, n);
        }
    
        delete[](buffer);
    }

    void Socket::InternalOnRecieve(char* str, int length)
    {
        if(OnRecieve == NULL)
            return;
        str[length] = '\0';
        OnRecieve(this, str, length);
    }

    void Socket::Activate()
    {
        if(active)
            throw "ERROR: trying to make a socket active, that already is active";
        if(!connected)
            throw "ERROR: trying to make an unconnected socket active";
        active = true;
        if(pthread_create(&monitor, NULL, MonitorSocket, this))
            throw "ERROR making socket active (could not create thread)";
    }
    
    void Socket::Deactivate()
    {
        if(!active)
            return;
        active = false;
        pthread_join(monitor, NULL);
    }


    // ---------------------------------------------------------- implementation

 
    Socket::Socket()
    {
        connected = false;
        active = false;
        OnRecieve = NULL;
        socket_handle = socket(AF_INET, SOCK_STREAM, 0);
        if(socket_handle < 0) 
            throw "ERROR creating socket";
    }

    Socket::Socket(string server)
    {
        connected = false;
        active = false;
        OnRecieve = NULL;
        socket_handle = socket(AF_INET, SOCK_STREAM, 0);
        if(socket_handle < 0) 
            throw "ERROR creating socket";
        Connect(server);
    }

    Socket::Socket(string& server, int port)
    {
        connected = false;
        active = false;
        OnRecieve = NULL;
        socket_handle = socket(AF_INET, SOCK_STREAM, 0);
        if(socket_handle < 0) 
            throw "ERROR creating socket";
        Connect(server, port);
    }

    Socket::~Socket()
    {
        Disconnect();
    }
    
    void Socket::Disconnect()
    {
        Deactivate();
        if(connected)
            close(socket_handle);
        connected = false;
    }

    void Socket::Connect(string& server)
    {
        int port = defaultport;
        for(int i = server.size()-1; i >= 0; i--)
        {
            if(server[i] == ':')
            {
                port = atoi(server.c_str()+i+1);
                server.resize(i);
                break;
            }
            if(server[i] < '0' || server[i] > '9')
                break;
        }
        Connect(server, port);
    }

    void Socket::Connect(string& server, int port)
    {
        Disconnect();
        in_addr_t IP = inet_addr(server.c_str());
        if(IP == -1)
        {
            struct hostent *pserver = gethostbyname(server.c_str());
            if(pserver == NULL)
                throw "ERROR resolving hostname";

            memcpy(&(server_address.sin_addr.s_addr), (pserver->h_addr), sizeof(char)*(pserver->h_length));
        }
        else
        {
            server_address.sin_addr.s_addr = inet_addr(server.c_str());
        }
        server_address.sin_port = htons(port);
        server_address.sin_family = AF_INET;

        if(connect(socket_handle,(struct sockaddr*)(&server_address),sizeof(server_address)) < 0) 
            throw "ERROR connecting to server";
        connected = true;
    }

    void Socket::Send(const string& message)
    {
        if(!connected)
            throw "ERROR trying to send a message over an unconnected socket";
        int n = write(socket_handle, message.c_str(), message.size());
        if(n < 0)
            throw "ERROR sending message";
    }

    string Socket::Recieve(int max_chars)
    {
        if(!connected)
            throw "ERROR trying to recieve a message from an unconnected socket";
        string result(max_chars+1, ' ');
        int n = read(socket_handle, &result[0], max_chars);
        if(n < 0)
            throw "ERROR reading from socket";
        result[n] = '\0';
        return result;
    }
    
    string Socket::RemoteIP()
    {
        if(!connected)
            throw "ERROR can\'t determine remote IP from an unconnected socket";
        return inet_ntoa(server_address.sin_addr);
    }

}